﻿Type=Service
Version=5.8
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
	#StartCommandReturnValue: android.app.Service.START_STICKY
#End Region

Sub Process_Globals
	Dim OW As OverlayWindow
	Dim MaxWidth, MaxHeight As Int
	Dim Cache As Cache
	Dim OBtn_Open, OBtn_Close As OverlayButton
	Dim ChangeColorTimer As Timer
	Dim IgnoreClipboardChange As Boolean
End Sub

Sub Service_Create
	Cache.Initialize(0, 100 * 1024, "") '100 Kb
	CreateInterface
	SetClipListener
	ChangeColorTimer.Initialize("CC", 1000) '1 second
	IgnoreClipboardChange = False
End Sub

Sub Service_Start (StartingIntent As Intent)
End Sub

Sub Service_Destroy
	'Closes the window and all buttons
	If OW.IsInitialized Then
		OBtn_Open.Remove
		OW.Close 'closes also OBtn_Close which is bound
	End If

	'Closes the disk cache
	Cache.CloseDiskCache
End Sub

#if JAVA
@Override
public void onConfigurationChanged(android.content.res.Configuration newConfig) {
    super.onConfigurationChanged(newConfig);
    processBA.raiseEvent(null, "orientation_changed");
}
#end if

Sub SetClipListener
	'Creates a listener for the clipboard change event
	Dim JO As JavaObject
	JO.InitializeContext
	JO = JO.RunMethodJO("getSystemService", Array As Object("clipboard"))
    Dim Listener As Object = JO.CreateEvent("android.content.ClipboardManager.OnPrimaryClipChangedListener", "Clipboard", Null)
    JO.RunMethod("addPrimaryClipChangedListener", Array(Listener))
End Sub

Sub Clipboard_Event(MethodName As String, Args() As Object)
	'Ignores the change if it is due to this application (when a list entry is selected)
	If IgnoreClipboardChange Then Return

	'Stores the new clipboard text in the disk cache
	Dim CB As BClipboard
	Dim Text As String = CB.getText
	If Text = "" Then Return
	Dim HashKey As Long = 0
	For i = 0 To Text.Length - 1
		HashKey = 151 * HashKey + (Asc(Text.CharAt(i)) - 32)
	Next
	If Not(Cache.IsOnDisk(HashKey)) Then
		Cache.PutStringOnDisk(HashKey, Text)

		'Adds the text to the ListView
		Dim lv As ListView = OW.Panel.GetView(0)
		lv.AddTwoLines(Text, Text.Replace(CRLF, "_"))

		'Warns the user that the text was stored succesfully by changing the color of the Open button
		ChangeColorOpenBtn(True)
		ChangeColorTimer.Enabled = True
	End If
End Sub

Sub ChangeColorOpenBtn(Highlight As Boolean)
	'Changes the color of the Open button
	Dim DrwOpenUp, DrwOpenDown As BitmapDrawable
	If Highlight Then
		DrwOpenUp.Initialize(LoadBitmap(File.DirAssets, "open_hilite.png"))
	Else
		DrwOpenUp.Initialize(LoadBitmap(File.DirAssets, "open.png"))
	End If
	DrwOpenDown.Initialize(LoadBitmap(File.DirAssets, "open_pressed.png"))
	OBtn_Open.SetStateImages(DrwOpenUp, DrwOpenDown, Null)
End Sub

Sub CreateInterface
	'Creates an overlay window with a ListView and two buttons (one to open the list, one to close the window)
	MaxWidth = GetDeviceLayoutValues.Width
	MaxHeight = GetDeviceLayoutValues.Height

	OW.Initialize(MaxWidth * 0.1, MaxHeight * 0.2, MaxWidth * 0.9, MaxHeight * 0.6, "")
	OW.Panel.Color = Colors.DarkGray
	OW.Open

	Dim lv As ListView
	lv.Initialize("lv")
	lv.TwoLinesLayout.ItemHeight = 53dip
	lv.TwoLinesLayout.Label.Height = 0 'The first label is not displayed; it holds the text in its native format
	lv.TwoLinesLayout.SecondLabel.Top = 0
	lv.TwoLinesLayout.SecondLabel.Height = lv.TwoLinesLayout.ItemHeight
	lv.TwoLinesLayout.SecondLabel.Gravity = Gravity.LEFT + Gravity.TOP
	lv.TwoLinesLayout.SecondLabel.TextSize = 14
	lv.TwoLinesLayout.SecondLabel.TextColor = Colors.White
	OW.Panel.AddView(lv, 0, 0, -1, -1)

	OBtn_Open.Initialize(MaxWidth - 30dip, OW.Y, 30dip, 60dip, "BtnOpen")
	ChangeColorOpenBtn(False)
	OBtn_Open.Add

	OBtn_Close.Initialize(MaxWidth - 40dip, OW.Y + OW.Height, 40dip, 40dip, "BtnClose")
	Dim DrwUp, DrwDown As BitmapDrawable
	DrwUp.Initialize(LoadBitmap(File.DirAssets, "close.png"))
	DrwDown.Initialize(LoadBitmap(File.DirAssets, "close_pressed.png"))
	OBtn_Close.SetStateImages(DrwUp, DrwDown, Null)
	OBtn_Close.BindTo(OW)
	OBtn_Close.Add

	OW.Visible = False

	'Fills the ListView with the contents of the disk cache	
	Dim mapCache As Map = Cache.DiskList
	For Each CacheEntryName As String In mapCache.Keys
		If CacheEntryName.EndsWith(".0") Then
			Dim Text As String = File.ReadString("", CacheEntryName)
			lv.AddTwoLines(Text, Text.Replace(CRLF, "_"))
		End If
	Next
End Sub

Sub CC_Tick
	'Reverts the color of the Open button
	ChangeColorOpenBtn(False)
	ChangeColorTimer.Enabled = False
End Sub

Sub UpdatePosition
	'Changes the size and position of views to fit the new screen orientation
	If OW.Visible Then
		OW.SetPosition(MaxWidth * 0.1, MaxHeight * 0.2)
		OW.SetSize(MaxWidth * 0.9, MaxHeight * 0.6)
		OBtn_Close.SetPosition(MaxWidth - 40dip, OW.Y + OW.Height)
	Else
		OBtn_Open.X = MaxWidth - 30dip
	End If
End Sub

Sub Orientation_Changed
	MaxWidth = GetDeviceLayoutValues.Width
	MaxHeight = GetDeviceLayoutValues.Height
	UpdatePosition
End Sub

Sub BtnOpen_Click
	OBtn_Open.Remove
	OW.Visible = True
	UpdatePosition
End Sub

Sub Close_List
	OW.Visible = False
	OBtn_Open.Add
	UpdatePosition
End Sub

Sub lv_ItemClick(Position As Int, Value As Object)
	'Copies the selected text into the clipboard
	IgnoreClipboardChange = True
	Dim CB As BClipboard
	CB.setText(Value)
	IgnoreClipboardChange = False
	Close_List
End Sub

Sub BtnClose_Click
	StopService(Me)
End Sub
